import { Component } from "@angular/core";
import { AddEditMasterComponent } from "../add-edit-master/add-edit-master.component";
import { TableApiService } from "src/app/_services/table-api.service";
import { NgbModal } from "@ng-bootstrap/ng-bootstrap";
import { Router } from "@angular/router";
import { BuildingService } from "../../service/monitoring-api.service";
import { LoginService } from "../../service/login.service";
import { ToastrService } from "ngx-toastr";

@Component({
  selector: "app-master-floor",
  templateUrl: "./master-floor.component.html",
  styleUrls: ["./master-floor.component.css"],
})
export class MasterFloorComponent {
  data: any;
  filteredRows: any[];
  searchTerm: string = "";
  rows: any = [];
  public breadcrumb: any;
  dataMasterCategori: any;

  constructor(
    private tableApiservice: TableApiService,
    private modalService: NgbModal,
    private router: Router,
    private monitoringApiService: BuildingService,
    private authService: LoginService,
    private toastr: ToastrService
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.breadcrumb = {
      mainlabel: "Master Floor",
      links: [
        {
          name: "Home",
          isLink: false,
        },
        {
          name: "Master Floor",
          isLink: false,
        },
      ],
    };
    this.fetchData();
  }

  fetchData() {
    this.monitoringApiService.getMasterData().subscribe((res) => {
      this.data = res.results.data;
      this.dataMasterCategori = res.results.data.find(
        (item) => item.name === "master_floor"
      );
      // this.filteredRows = this.dataMasterCategori.headerDetailParam;
      this.filteredRows = this.dataMasterCategori.headerDetailParam.sort(
        (a, b) => b.id - a.id
      );
    });
  }

  filterRows() {
    if (!this.searchTerm) {
      this.filteredRows = [
        ...this.data.find((item) => item.name === "master_floor")
          .headerDetailParam,
      ];
    } else {
      this.filteredRows = this.data
        .find((item) => item.name === "master_floor")
        .headerDetailParam.filter((row) => this.rowContainsSearchTerm(row));
    }
  }

  rowContainsSearchTerm(row: any): boolean {
    const searchTermLC = this.searchTerm.toLowerCase();
    return Object.values(row).some(
      (value) =>
        value !== null && value.toString().toLowerCase().includes(searchTermLC)
    );
  }

  openAddMasterModal() {
    const modalRef = this.modalService.open(AddEditMasterComponent, {
      size: "lg",
      backdrop: "static", // Add this line
      keyboard: false, // Add this line
    });

    modalRef.componentInstance.headerId = this.dataMasterCategori.id;
    modalRef.componentInstance.mode = "add";
    modalRef.componentInstance.floor = true;
    modalRef.result.then(
      (result) => {
        console.log(result);
        if (result) {
          this.monitoringApiService
            .postHeaderDetailParam(result)
            .subscribe((res) => {
              this.fetchData();
              if (res.status === 400) {
                this.toastr.error("Warning", "Data yang anda masukan double.", {
                  timeOut: 5000,
                  closeButton: true,
                });
              }
            });
        }
        // if (result) {
        //   this.monitoringApiService
        //     .postHeaderDetailParam(result)
        //     .subscribe((res) => {
        //       console.log(res);
        //       this.fetchData();
        //     });
        // }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
  }

  editRow(row) {
    const modalRef = this.modalService.open(AddEditMasterComponent, {
      size: "lg",
      backdrop: "static", // Add this line
      keyboard: false, // Add this line
    });

    modalRef.componentInstance.headerId = this.dataMasterCategori.id;
    modalRef.componentInstance.dataRow = row;
    modalRef.componentInstance.mode = "edit";
    modalRef.componentInstance.floor = true;
    modalRef.result.then(
      (result) => {
        if (result) {
          this.monitoringApiService
            .putHeaderDetailParam(result, row.id)
            .subscribe((res) => {
              this.fetchData();
              if (res.status === 400) {
                this.toastr.error("Warning", "Data yang anda masukan double.", {
                  timeOut: 5000,
                  closeButton: true,
                });
              }
            });
        }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
  }

  deleteRow(row) {
    const confirmDelete = confirm("Are you sure you want to delete this item?");
    if (confirmDelete) {
      this.monitoringApiService.deleteHeaderDetailParam(row.id).subscribe(
        (res) => {
          this.fetchData();
          this.toastr.success("Success", "Delete Completed.", {
            timeOut: 2000,
            closeButton: true,
          });
        },
        (error) => {
          console.error(error);
          this.toastr.error("Error", "Data sedang digunakan!", {
            timeOut: 2000,
            closeButton: true,
          });
        }
      );
    }
  }
}
