import { Component, ElementRef, ViewChild } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { ChartOptions, ChartType, ChartDataset } from "chart.js";
import { EnergyMonitoringService } from "../../service/energy-monitoring.service";
import { CurrencyPipe } from "@angular/common";
import { LoginService } from "../../service/login.service";
import { DeviceService } from "../../service/device.service";
import { color } from "echarts";
import { NgbModal } from "@ng-bootstrap/ng-bootstrap";
import { ModalExportComponent } from "../modal-export/modal-export.component";

@Component({
  selector: "app-detail",
  templateUrl: "./detail.component.html",
  styleUrls: ["./detail.component.css"],
  providers: [CurrencyPipe],
})
export class DetailComponent {
  data: any;
  mode: string;
  breadcrumb: any;
  donutChart1: any;
  donutChart2: any;

  // temperature
  days = [
    { name: "Sun", isToday: false },
    { name: "Mon", isToday: false },
    { name: "Tue", isToday: false },
    { name: "Wed", isToday: false },
    { name: "Thu", isToday: false },
    { name: "Fri", isToday: false },
    { name: "Sat", isToday: false },
  ];
  roomTemperature: number = 0;
  roomHumidity: number = 0;
  valueAirQuality: number = 0;
  currentDate: string;
  currentTime: string;
  chartOption: any;
  chartOptionBar: any;
  chartInstance: any;

  // chart bar
  public barChartLabels: string[] = [];
  public barChartData: number[] = [];
  public barChartData2: number[] = [];
  //..........................

  // integrasi
  electric: any;
  water: any;
  device: any;
  room: any;
  topCard: any;
  summaryCost: any;
  temperature: any;
  deviceCategory: any;
  chartKwhWater: any;
  buildingId: any;
  floorId: any;
  roomId: any;
  //......

  constructor(
    private route: ActivatedRoute,
    private energyMonitoringService: EnergyMonitoringService,
    private currencyPipe: CurrencyPipe,
    private router: Router,
    private authService: LoginService,
    private modalService: NgbModal,
  ) {}

  get formattedSummaryCost(): string {
    return this.currencyPipe.transform(
      this.summaryCost.summary_cost,
      "IDR",
      "symbol",
      "1.0-0"
    );
  }

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.route.data.subscribe((data) => {
      this.mode = data.mode;
    });
    this.breadcrumbLink();
    this.route.params.subscribe((params) => {

      this.buildingId = params["buildingId"] ? params["buildingId"] : 0;
      this.floorId = params["floorId"] ? params["floorId"] : 0;
      this.roomId = params["roomId"] ? params["roomId"] : 0;

      if (this.buildingId !== undefined && this.floorId === undefined && this.roomId === undefined) {
        this.dataEnergyMonitoringTopCard(this.buildingId, 0, 0);
        this.dataEnergyMonitoringSummary(this.buildingId, 0, 0);
        this.dataEnergyDeviceCategory(this.buildingId, 0, 0);
        this.dataEnergyChartKwhWater(this.buildingId, 0, 0);
      } else if (this.buildingId !== undefined && this.floorId !== undefined && this.roomId === undefined) {
        this.dataEnergyMonitoringTopCard(this.buildingId, this.floorId, 0);
        this.dataEnergyMonitoringSummary(this.buildingId, this.floorId, 0);
        this.dataEnergyDeviceCategory(this.buildingId, this.floorId, 0);
        this.dataEnergyChartKwhWater(this.buildingId, this.floorId, 0);
      } else if (this.buildingId !== undefined && this.floorId !== undefined && this.roomId !== undefined) {
        this.dataEnergyMonitoringTopCard(this.buildingId, this.floorId, this.roomId);
        this.dataEnergyMonitoringSummary(this.buildingId, this.floorId, this.roomId);
        this.dataEnergyDeviceCategory(this.buildingId, this.floorId, this.roomId);
        this.dataEnergyChartKwhWater(this.buildingId, this.floorId, this.roomId);
      }
      
      this.dataDeviceIr();
      this.getAirQualityData();
    });
  }

  getAirQualityData() {
    this.valueAirQuality = Math.floor(Math.random() * 501);
    this.chartOption = {
      series: [
        {
          type: "gauge",
          startAngle: 180,
          endAngle: 0,
          center: ["50%", "75%"],
          radius: "130%", // Adjust the radius to make the chart smaller
          min: 0,
          max: 500,
          splitNumber: 5,
          axisLine: {
            lineStyle: {
              width: 20,
              color: [
                [1 / 6, "#6BD67C"], // 0 - 50: Green
                [2 / 6, "#A2D856"], // 51 - 100: Light Green
                [3 / 6, "#FFE45E"], // 101 - 150: Yellow
                [4 / 6, "#FFA644"], // 151 - 200: Orange
                [5 / 6, "#FF6E76"], // 201 - 300: Light Red
                [6 / 6, "#E768A7"], // 301 - 500: Purple
              ],
            },
          },
          pointer: {
            icon: "path://M12.8,0.7l12,40.1H0.7L12.8,0.7z",
            length: "12%",
            width: 15,
            offsetCenter: [0, "-50%"],
            itemStyle: {
              color: "auto",
            },
          },
          axisTick: {
            length: 10,
            lineStyle: {
              color: "auto",
              width: 2,
            },
          },
          splitLine: {
            length: 15,
            lineStyle: {
              color: "auto",
              width: 5,
            },
          },
          axisLabel: {
            color: "#464646",
            fontSize: 16, // Adjust font size
            distance: -40, // Adjust distance
            rotate: "tangential",
            formatter: function (value: number) {
              // return Math.round(value) === 100 ? "100" : Math.round(value) + "";
              return value.toString();
            },
          },
          title: {
            offsetCenter: [0, " 15%"],
            fontSize: 18, // Adjust font size
          },
          detail: {
            fontSize: 24, // Adjust font size
            offsetCenter: [0, "-25%"], // Adjust offset
            valueAnimation: true,
            formatter: function (value: number) {
              return Math.round(value) + "";
            },
            color: "#242222",
          },
          data: [
            {
              value: this.valueAirQuality,
              name: this.getGradeName(this.valueAirQuality),
            },
          ],
        },
      ],
    };
  }

  getGradeName(value: number): string {
    if (value <= 50) {
      return "Good";
    } else if (value <= 100) {
      return "Moderate";
    } else if (value <= 150) {
      return "Unhealthy for Sensitive Groups";
    } else if (value <= 200) {
      return "Unhealthy";
    } else if (value <= 300) {
      return "Very Unhealthy";
    } else {
      return "Hazardous";
    }
  }

  breadcrumbLink() {
    this.breadcrumb = {
      mainlabel: "Energy Monitoring",
      linkBack: "/monitoring",
      isLinkBack: true,
      links: [
        {
          name: "Dashboard",
          isLink: false,
          link: "",
        },
        {
          name: "Energy Monitoring",
          isLink: false,
          link: "",
        },
      ],
    };
  }

  dataDeviceIr() {
    const today = new Date().getDay();
    this.days[today].isToday = true;
    const now = new Date();
    this.currentDate = now.toLocaleDateString();
    this.currentTime = now.toLocaleTimeString();
    this.energyMonitoringService.getDeviceStatus(452).subscribe((res) => {
      this.roomTemperature = res.result[0].value;
      this.roomHumidity = res.result[1].value;
    });
  }
  //integrasi
  dataEnergyMonitoringTopCard(buildingId, floorId, roomId) {
    this.energyMonitoringService
      .getDashboardTopCard(buildingId, floorId, roomId)
      .subscribe((res) => {
        this.topCard = res.data;
      });
  }

  dataEnergyMonitoringSummary(buildingId, floorId, roomId) {
    this.energyMonitoringService
      .getDashboardSummary(buildingId, floorId, roomId)
      .subscribe((res) => {
        this.summaryCost = res.data;
      });
  }

  dataEnergyDeviceCategory(buildingId, floorId, roomId) {
    this.energyMonitoringService
      .getDashboardDeviceCategory(buildingId, floorId, roomId)
      .subscribe((res) => {
        this.deviceCategory = res.data[0];
      });
  }

  dataEnergyChartKwhWater(buildingId, floorId, roomId) {
    this.energyMonitoringService
      .getDashboardChartKwhWater(buildingId, floorId, roomId)
      .subscribe((res) => {
        this.chartKwhWater = res.data;
        this.chartKwhWater.forEach((entry) => {
          this.barChartData.push(entry.kwh.toFixed(1));
          this.barChartData2.push(entry.cost);
          this.barChartLabels.push(entry.day);
        });

        this.chartOptionBar = {
          tooltip: {
            trigger: "axis",
            axisPointer: {
              type: "shadow",
            },
          },
          legend: {},
          toolbox: {
            show: true,
            orient: 'vertical',
            feature: {
              dataView: { show: true, readOnly: false },
              magicType: { show: true, type: ['line', 'bar'] },
              restore: { show: true },
              saveAsImage: { show: true }
            }
          },
          grid: {
            left: "3%",
            right: "4%",
            bottom: "3%",
            containLabel: true,
          },
          xAxis: [
            {
              type: "category",
              data: this.barChartLabels,
              
            },
          ],
          yAxis: [
            {
              type: "value",
            },
          ],
          series: [
            {
              name: "KWH Consumption",
              type: "bar",
              color: "#242222",
              stack: "Ad",
              emphasis: {
                focus: "series",
              },
              data: this.barChartData,
            },
            {
              name: "Summary Cost",
              type: "bar",
              stack: "Ad",
              color: "#37a647",
              emphasis: {
                focus: "series",
              },
              data: this.barChartData2,
            },
          ],
        };
        onClick: (params) => {
          this.onChartClick(params);
        }
      });
  }

  onChartClick(params) {
    console.log(params);
    const modalRef = this.modalService.open(ModalExportComponent, {
      size: "xl",
      centered: true,
    });
    modalRef.componentInstance.dataIndex = params.dataIndex;
    modalRef.componentInstance.buildingId = this.buildingId;
    modalRef.componentInstance.floorId = this.floorId;
    modalRef.componentInstance.roomId = this.roomId;
    modalRef.componentInstance.mode = this.mode;

    modalRef.result.then(
      (result) => {
        if (result) {
          console.log(result);
          
        }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
    
    // Show popup with details
    // alert(`Day: ${params.name}\nKWH Consumption: ${params.value}`);
  }

  seeMore(buildingId, floorId, roomId) {
    this.router.navigate(["/monitoring/control-device-see-more/", buildingId, floorId, roomId, this.mode]);
  }
}
